# ------------------------------------------------------------------------------
# Test google Format
# ------------------------------------------------------------------------------
import sys
import datetime as dt
import pytz
from django.contrib.auth.models import User
from django.test import TestCase
from wagtail.core.models import Site, Page
from ls.joyous.models.calendar import CalendarPage
from ls.joyous.models import (SimpleEventPage, MultidayEventPage,
        RecurringEventPage)
from ls.joyous.utils.recurrence import Recurrence
from ls.joyous.utils.recurrence import MONTHLY, FR
from ls.joyous.formats.google import (SimpleGEvent, MultidayGEvent,
        RecurringGEvent)

# ------------------------------------------------------------------------------
class TestSimple(TestCase):
    def setUp(self):
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@joy.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()

    def testFromPage(self):
        page = SimpleEventPage(owner = self.user,
                               slug  = "baseball-game",
                               title = "Baseball Game",
                               date  = dt.date(2014,7,30),
                               time_from = dt.time(13),
                               time_to   = dt.time(16),
                               tz = pytz.timezone("America/Los_Angeles"))
        self.calendar.add_child(instance=page)
        page.save_revision().publish()
        gev = SimpleGEvent.fromPage(page)
        self.assertEqual(gev['dates'], "20140730T130000/20140730T160000")
        self.assertEqual(gev['ctz'],   "America/Los_Angeles")

# ------------------------------------------------------------------------------
class TestMultiday(TestCase):
    def setUp(self):
        site = Site.objects.get(is_default_site=True)
        site.hostname = "joy.test"
        site.save()
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@joy.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()

    def testFromPage(self):
        page = MultidayEventPage(owner = self.user,
                                 slug  = "kainiue-food-festival",
                                 title = "KaiNiue Food Festival",
                                 date_from = dt.date(2018,11,2),
                                 date_to   = dt.date(2018,11,5),
                                 tz = pytz.timezone("Pacific/Niue"))
        self.calendar.add_child(instance=page)
        page.save_revision().publish()
        gev = MultidayGEvent.fromPage(page)
        self.assertEqual(gev['dates'], "20181102T000000/20181105T235959")
        self.assertEqual(gev['ctz'],   "Pacific/Niue")

# ------------------------------------------------------------------------------
class TestRecurring(TestCase):
    def setUp(self):
        site = Site.objects.get(is_default_site=True)
        site.hostname = "joy.test"
        site.save()
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@joy.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()

    def testFromPage(self):
        page = RecurringEventPage(owner   = self.user,
                                  slug    = "lunch-and-code",
                                  title   = "Lunch and Code",
                                  details = "What we'll do really depends on "
                                            "what the group feels is useful, "
                                            "some ideas are:\n"
                                            "- Providing feedback on past, or "
                                            "in progress, projects\n"
                                            "- Pair programming\n"
                                            "- Coding\n",
                                  repeat    = Recurrence(dtstart=dt.date(2017,3,15),
                                                         freq=MONTHLY,
                                                         byweekday=[FR(-1)]),
                                  time_from = dt.time(12),
                                  time_to   = dt.time(13),
                                  tz = pytz.timezone("Pacific/Auckland"),
                                  location  = "70 Molesworth Street, Wellington")
        self.calendar.add_child(instance=page)
        page.save_revision().publish()
        gev = RecurringGEvent.fromPage(page)
        self.assertEqual(gev.url,
                         "http://www.google.com/calendar/event?"
                         "action=TEMPLATE&"
                         "text=Lunch+and+Code&"
                         "details=What+we%27ll+do+really+depends+on+what+the+"
                         "group+feels+is+useful%2C+some+ideas+are%3A%0A"
                         "-+Providing+feedback+on+past%2C+or+in+progress%2C+projects%0A"
                         "-+Pair+programming%0A-+Coding%0A&"
                         "location=70+Molesworth+Street%2C+Wellington&"
                         "dates=20170331T120000%2F20170331T130000&"
                         "ctz=Pacific%2FAuckland&"
                         "recur=RRULE%3AFREQ%3DMONTHLY%3BWKST%3DSU%3BBYDAY%3D-1FR")

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Middleware
# ------------------------------------------------------------------------------
import pytz
from django.utils import timezone
from django.utils.deprecation import MiddlewareMixin


class UserTimeZoneMiddleware(MiddlewareMixin):
    """
    If there is a Wagtail user with their time zone set then activate
    that time zone for all pages (not just the Wagtail Admin).
    """
    def process_request(self, request):
        try:
            tzname = request.user.wagtail_userprofile.current_time_zone
        except AttributeError:
            tzname = None
        if tzname:
            timezone.activate(pytz.timezone(tzname))
        else:
            timezone.deactivate()



# ------------------------------------------------------------------------------
# Test Edit Handlers
# ------------------------------------------------------------------------------
import sys
import datetime as dt
from unittest import skipUnless
from django.test import RequestFactory, TestCase, override_settings
from django.conf import settings
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.formats import get_format
from wagtail.admin.edit_handlers import get_form_for_model
from wagtail.admin.widgets import AdminTimeInput, AdminDateInput
from wagtail.core.models import Site, Page
from ls.joyous.models.events import (CancellationPageForm, RecurringEventPageForm,
                                     HiddenNumDaysPanel)
from ls.joyous.models import (CalendarPage, CancellationPage, RecurringEventPage,
                              MultidayRecurringEventPage)
from ls.joyous.utils.recurrence import Recurrence, MONTHLY, YEARLY, TU, FR
from ls.joyous.edit_handlers import ExceptionDatePanel, ConcealedPanel
from ls.joyous.widgets import Time12hrInput, ExceptionDateInput
from .testutils import datetimetz, freeze_timetz, getPage
import ls.joyous.edit_handlers
import importlib
from wagtail import VERSION as _wt_version
WagtailVersion = _wt_version[:3]

# ------------------------------------------------------------------------------
class TestExceptionDatePanel(TestCase):
    def setUp(self):
        self.home = getPage("/home/")
        self.user = User.objects.create_superuser('i', 'i@joy.test', 's3(r3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(slug      = "leaders-meeting",
                                        title     = "Leaders' Meeting",
                                        repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(3)]),
                                        time_from = dt.time(19),
                                        tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def _getRequest(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        request.site = Site.objects.get(is_default_site=True)
        return request

    def testWidget(self):
        self.assertIs(ExceptionDatePanel.widget, ExceptionDateInput)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testBindWithoutForm25(self):
        cancellation = CancellationPage(owner = self.user,
                                        except_date = dt.date(2019,1,21))
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        self.assertIsNone(panel.form)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testBindWithoutOverrides25(self):
        cancellation = CancellationPage(owner = self.user,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        panel = panel.bind_to(request=self._getRequest())
        panel = panel.bind_to(form=form)
        self.assertIsNotNone(panel.form)
        self.assertIsNone(panel.instance.overrides)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testBindOverridesRepeat25(self):
        cancellation = CancellationPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        widget = form['except_date'].field.widget
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        panel = panel.bind_to(request=self._getRequest())
        panel = panel.bind_to(form=form)
        self.assertIs(widget.overrides_repeat, self.event.repeat)
        self.assertIsNone(panel.exceptionTZ)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    @timezone.override("America/Los_Angeles")
    def testBindExceptionTZ25(self):
        cancellation = CancellationPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        panel = panel.bind_to(request=self._getRequest())
        panel = panel.bind_to(form=form)
        self.assertEqual(panel.exceptionTZ, "Asia/Tokyo")

# ------------------------------------------------------------------------------
@override_settings(JOYOUS_TIME_INPUT=12)
class TestTime12hrPanel(TestCase):
    def testWidget(self):
        importlib.reload(ls.joyous.edit_handlers)
        from ls.joyous.edit_handlers import TimePanel
        self.assertIs(TimePanel.widget, Time12hrInput)

    def testDefaultTimeInput(self):
        importlib.reload(ls.joyous.edit_handlers)
        self.assertIn('%I:%M%p', settings.TIME_INPUT_FORMATS)
        self.assertIn('%I%p', settings.TIME_INPUT_FORMATS)

    @override_settings(LANGUAGE_CODE="en-nz")
    def testNZLocaleTimeInput(self):
        importlib.reload(ls.joyous.edit_handlers)
        format = get_format('TIME_INPUT_FORMATS')
        self.assertIn('%I:%M%p', format)
        self.assertIn('%I%p', format)


# ------------------------------------------------------------------------------
class TestConcealedPanel(TestCase):
    def setUp(self):
        self.home = getPage("/home/")
        self.user = User.objects.create_superuser('i', 'i@joy.test', 's3(r3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(slug      = "leaders-meeting",
                                        title     = "Leaders' Meeting",
                                        repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(3)]),
                                        time_from = dt.time(19),
                                        tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def _getRequest(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        request.site = Site.objects.get(is_default_site=True)
        return request

    def testInit(self):
        panel = ConcealedPanel([], "Test", help_text="Nothing")
        self.assertEqual(panel._heading, "Test")
        self.assertEqual(panel._help_text, "Nothing")
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testConcealed25(self):
        panel = ConcealedPanel([], "Test")
        panel = panel.bind_to(instance=self.event)
        panel = panel.bind_to(request=self._getRequest())
        content = panel.render()
        self.assertEqual(content, "")
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testShown25(self):
        class ShownPanel(ConcealedPanel):
            def _show(self):
                return True

        panel = ShownPanel([], "Test", help_text="Nothing")
        panel = panel.bind_to(instance=self.event)
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")
        panel = panel.bind_to(request=self._getRequest())
        content = panel.render()
        self.assertHTMLEqual(content, """
<fieldset>
    <legend>Test</legend>
    <ul class="fields">
    </ul>
</fieldset>
""")
        self.assertEqual(panel.heading, "Test")
        self.assertEqual(panel.help_text, "Nothing")

# ------------------------------------------------------------------------------
class TestHiddenNumDaysPanel(TestCase):
    FIELD_CONTENT = """
<fieldset>
  <legend>Number of days</legend>
  <ul class="fields">
    <li>
      <div class="field integer_field number_input ">
        <div class="field-content">
          <div class="input  ">
            <input type="number" name="num_days" value="1" required id="id_num_days">
            <span></span>
          </div>
        </div>
      </div>
    </li>
  </ul>
</fieldset>
"""

    def setUp(self):
        self.home = getPage("/home/")
        self.user = User.objects.create_superuser('i', 'i@joy.test', 's3(r3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(slug      = "leaders-meeting",
                                        title     = "Leaders' Meeting",
                                        repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(3)]),
                                        time_from = dt.time(19),
                                        tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()
        Form = get_form_for_model(RecurringEventPage,
                                  form_class=RecurringEventPageForm)
        self.form = Form(instance=self.event, parent_page=self.calendar)

    def _getRequest(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        request.site = Site.objects.get(is_default_site=True)
        return request

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testHidden25(self):
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to(instance=self.event,
                              request=self._getRequest(),
                              form=self.form)
        content = panel.render_as_object()
        self.assertEqual(content, "")
        content = panel.render_as_field()
        self.assertEqual(content, "")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testShowWith2Days25(self):
        self.event.num_days = 2
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to(instance=self.event,
                              request=self._getRequest(),
                              form=self.form)
        content = panel.render_as_object()
        self.assertHTMLEqual(content, self.FIELD_CONTENT)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testShowMulidayRecurringEvent25(self):
        event = MultidayRecurringEventPage(slug      = "leaders-retreat",
                                           title     = "Leaders' Retreet",
                                           repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                                  freq=YEARLY,
                                                                  bymonth=3,
                                                                  byweekday=[FR(1)]),
                                           time_from = dt.time(19),
                                           num_days  = 3,
                                           tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=event)
        event.save_revision().publish()
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to(instance=event,
                              request=self._getRequest(),
                              form=self.form)
        content = panel.render_as_object()
        self.assertHTMLEqual(content, self.FIELD_CONTENT)

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test ical utility clases
# ------------------------------------------------------------------------------
import sys
import datetime as dt
import pytz
from django.test import TestCase
from django.utils import timezone
from wagtail.core.models import Page
from icalendar import vDatetime, vDate, vRecur, vDDDTypes, vText
from ls.joyous.utils.telltime import getLocalDatetime
from ls.joyous.formats.ical import (VResults, vDt, vSmart, TimeZoneSpan, VMatch,
                                    VEventFactory, CalendarTypeError,
                                    Event, VEvent, SimpleVEvent)
from freezegun import freeze_time

# ------------------------------------------------------------------------------
class TestVResults(TestCase):
    def testEmpty(self):
        results = VResults()
        self.assertEqual(results.success, 0)
        self.assertEqual(results.fail,    0)
        self.assertEqual(results.error,   0)

    def testInit(self):
        results = VResults(3, 2, 1)
        self.assertEqual(results.success, 3)
        self.assertEqual(results.fail,    2)
        self.assertEqual(results.error,   1)

    def testEquals(self):
        a = VResults()
        b = VResults()
        self.assertTrue(a == b)
        b.success = 1
        self.assertFalse(a == b)
        a += b
        self.assertTrue(a == b)
        b.error = 1
        self.assertFalse(a == b)

    def testRepr(self):
        results = VResults()
        self.assertEqual(repr(results), "Success=0, Fail=0, Error=0")
        results.error = 1
        self.assertEqual(repr(results), "Success=0, Fail=0, Error=1")

# ------------------------------------------------------------------------------
class TestVDt(TestCase):
    def testNone(self):
        v = vDt()
        self.assertFalse(v)
        self.assertEqual(v, None)
        self.assertEqual(v.date(), None)
        self.assertEqual(v.time(), None)
        self.assertEqual(v.datetime(), None)
        self.assertEqual(v.tzinfo(), None)
        self.assertEqual(v.zone(), None)
        self.assertEqual(v.timezone(), pytz.timezone("Asia/Tokyo"))

    def testNaiveDt(self):
        mo = dt.datetime(1987, 6, 21, 3, 54, 0)
        v = vDt(mo)
        self.assertTrue(v)
        self.assertEqual(v, mo)
        self.assertEqual(v, vDatetime(mo))
        self.assertEqual(v, vDatetime.from_ical("19870621T035400"))
        self.assertEqual(v.date(), mo.date())
        self.assertEqual(v.time(), mo.time())
        self.assertEqual(v.datetime(), timezone.make_aware(mo))
        self.assertEqual(v.tzinfo(), None)
        self.assertEqual(v.zone(), None)
        self.assertEqual(v.timezone(), pytz.timezone("Asia/Tokyo"))

    def testTzinfoDt(self):
        mo = dt.datetime(2020, 2, 2, 2, tzinfo=dt.timezone.utc)
        v = vDt(mo)
        self.assertTrue(v)
        self.assertEqual(v, mo)
        self.assertEqual(v, vDatetime(mo))
        self.assertEqual(v, vDatetime.from_ical("20200202T020000Z"))
        self.assertEqual(v.date(), mo.date())
        self.assertEqual(v.time(), mo.time())
        self.assertEqual(v.datetime(), mo)
        self.assertEqual(v.tzinfo(), mo.tzinfo)
        self.assertEqual(v.zone()[:3], "UTC") # changed in Python 3.6
        self.assertEqual(v.timezone(), dt.timezone.utc)

    def testAwareDt(self):
        mo = vDatetime(timezone.make_aware(dt.datetime(2013, 4, 25, 6, 0),
                                           pytz.timezone("Pacific/Chatham")))
        v = vDt(mo)
        self.assertTrue(v)
        self.assertEqual(v, mo)
        self.assertEqual(v, mo.dt)
        self.assertEqual(v, vDatetime.from_ical("20130425T060000",
                                                "Pacific/Chatham"))
        self.assertEqual(v.date(), mo.dt.date())
        self.assertEqual(v.time(), mo.dt.time())
        self.assertEqual(v.datetime(), mo.dt)
        self.assertEqual(v.tzinfo(), mo.dt.tzinfo)
        self.assertEqual(v.zone(), "Pacific/Chatham")
        self.assertEqual(v.timezone(), pytz.timezone("Pacific/Chatham"))

    def testUnknownTZDt(self):
        tz = pytz.FixedOffset(540)
        tz.zone = "Japan/Edo"
        mo = timezone.make_aware(dt.datetime(2013, 4, 25, 6, 0), tz)
        v = vDt(mo)
        self.assertTrue(v)
        self.assertEqual(v, mo)
        self.assertEqual(v, vDatetime(mo))
        self.assertEqual(v.date(), mo.date())
        self.assertEqual(v.time(), mo.time())
        self.assertEqual(v.datetime(), mo)
        self.assertEqual(v.tzinfo(), mo.tzinfo)
        self.assertEqual(v.zone(), "Japan/Edo")
        with self.assertRaises(CalendarTypeError):
            v.timezone()

    def testDate(self):
        day = dt.date(1979, 8, 16)
        v = vDt(day)
        self.assertTrue(v)
        self.assertEqual(v, day)
        self.assertEqual(v, vDate(day))
        self.assertEqual(v, vDate.from_ical("19790816"))
        self.assertEqual(v.date(), day)
        self.assertEqual(v.time(), None)
        self.assertEqual(v.datetime(), getLocalDatetime(day, dt.time.min))
        self.assertEqual(v.tzinfo(), None)
        self.assertEqual(v.zone(), None)
        self.assertEqual(v.timezone(), pytz.timezone("Asia/Tokyo"))

    def testDateInc(self):
        day = dt.date(1979, 8, 16)
        v = vDt(day, inclusive=True)
        self.assertTrue(v)
        self.assertEqual(v, dt.date(1979, 8, 17))
        self.assertEqual(v.date(), dt.date(1979, 8, 17))
        self.assertEqual(v.date(inclusive=True), day)

# ------------------------------------------------------------------------------
class TestVSmart(TestCase):
    def testEmpty(self):
        v = vSmart("")
        self.assertFalse(v)
        self.assertEqual(str(v), "")

    def testStr(self):
        v = vSmart("ġedæġhwāmlīcan")
        self.assertTrue(v)
        self.assertEqual(str(v), "ġedæġhwāmlīcan")

    def testQuoPri(self):
        v = vSmart(b'=C4=A1ed=C3=A6=C4=A1hw=C4=81ml=C4=ABcan')
        v.params['ENCODING'] = "QUOTED-PRINTABLE"
        self.assertEqual(str(v), "ġedæġhwāmlīcan")

    def testBase64(self):
        v = vSmart(b'xKFlZMOmxKFod8SBbWzEq2Nhbg==')
        v.params['ENCODING'] = "BASE64"
        self.assertEqual(str(v), "ġedæġhwāmlīcan")

# ------------------------------------------------------------------------------
def addProps(vev, **kwargs):
    dtProps = {'dtstart', 'dtend', 'dtstamp', 'created', 'last-modified'}
    for prop, v in kwargs.items():
        if prop in dtProps:
            if not hasattr(v, 'dt'):
                v = vDt(v)
            vev.add(prop, v)
        else:
            vev.add(prop, v)

class TestTimeZoneSpan(TestCase):
    def testEmpty(self):
        span = TimeZoneSpan()
        with self.assertRaises(TimeZoneSpan.NotInitializedError):
            span.createVTimeZone(pytz.utc)

    def test1LunchTime(self):
        tz = pytz.timezone("Pacific/Auckland")
        vev = SimpleVEvent()
        addProps(vev,
                 summary = "Friday lunch",
                 uid = "1800",
                 dtstart = timezone.make_aware(dt.datetime(1996, 3, 1, 12), tz),
                 dtend   = timezone.make_aware(dt.datetime(1996, 3, 1, 13), tz))
        span = TimeZoneSpan(vev)
        vtz = span.createVTimeZone(tz)
        self.assertEqual(vtz['TZID'], "Pacific/Auckland")
        export = vtz.to_ical()
        nzdt = b"\r\n".join([
                 b"BEGIN:DAYLIGHT",
                 b"DTSTART;VALUE=DATE-TIME:19951001T030000",
                 b"TZNAME:NZDT",
                 b"TZOFFSETFROM:+1200",
                 b"TZOFFSETTO:+1300",
                 b"END:DAYLIGHT", ])
        self.assertIn(nzdt, export)
        nzst = b"\r\n".join([
                 b"BEGIN:STANDARD",
                 b"DTSTART;VALUE=DATE-TIME:19960317T020000",
                 b"TZNAME:NZST",
                 b"TZOFFSETFROM:+1300",
                 b"TZOFFSETTO:+1200",
                 b"END:STANDARD", ])
        self.assertIn(nzst, export)

    def testLunchTimes(self):
        tz = pytz.timezone("Pacific/Auckland")
        vev = SimpleVEvent()
        addProps(vev, summary = "All lunch times", uid = "999",
                 dtstart = timezone.make_aware(dt.datetime(1996, 3, 1, 12), tz),
                 dtend   = timezone.make_aware(dt.datetime(1996, 3, 1, 13), tz),
                 rrule   = vRecur.from_ical("FREQ=WEEKLY;WKST=SU;BYDAY=MO,TU,WE,TH,FR"))
        span = TimeZoneSpan(vev)
        vev = SimpleVEvent()
        addProps(vev, summary = "Saturday Brunch", uid = "2000",
                 dtstart = timezone.make_aware(dt.datetime(2019, 6, 4, 10), tz),
                 dtend   = timezone.make_aware(dt.datetime(2019, 6, 4, 11, 30), tz))
        span.add(vev)
        vtz = span.createVTimeZone(tz)
        self.assertEqual(vtz['TZID'], "Pacific/Auckland")
        export = vtz.to_ical()
        nzdt = b"\r\n".join([
                 b"BEGIN:DAYLIGHT",
                 b"DTSTART;VALUE=DATE-TIME:19951001T030000",
                 b"RDATE:19961006T030000,19971005T030000,19981004T030000,19991003T030000,2000",
                 b" 1001T030000,20011007T030000,20021006T030000,20031005T030000,20041003T03000",
                 b" 0,20051002T030000,20061001T030000,20070930T030000,20080928T030000,20090927",
                 b" T030000,20100926T030000,20110925T030000,20120930T030000,20130929T030000,20",
                 b" 140928T030000,20150927T030000,20160925T030000,20170924T030000,20180930T030",
                 b" 000,20190929T030000,20200927T030000,20210926T030000,20220925T030000,202309",
                 b" 24T030000,20240929T030000,20250928T030000,20260927T030000,20270926T030000,",
                 b" 20280924T030000,20290930T030000,20300929T030000,20310928T030000,20320926T0",
                 b" 30000,20330925T030000,20340924T030000,20350930T030000,20360928T030000,2037",
                 b" 0927T030000",
                 b"TZNAME:NZDT",
                 b"TZOFFSETFROM:+1200",
                 b"TZOFFSETTO:+1300",
                 b"END:DAYLIGHT", ])
        self.assertIn(nzdt, export)
        nzst = b"\r\n".join([
                 b"BEGIN:STANDARD",
                 b"DTSTART;VALUE=DATE-TIME:19960317T020000",
                 b"RDATE:19970316T020000,19980315T020000,19990321T020000,20000319T020000,2001",
                 b" 0318T020000,20020317T020000,20030316T020000,20040321T020000,20050320T02000",
                 b" 0,20060319T020000,20070318T020000,20080406T020000,20090405T020000,20100404",
                 b" T020000,20110403T020000,20120401T020000,20130407T020000,20140406T020000,20",
                 b" 150405T020000,20160403T020000,20170402T020000,20180401T020000,20190407T020",
                 b" 000,20200405T020000,20210404T020000,20220403T020000,20230402T020000,202404",
                 b" 07T020000,20250406T020000,20260405T020000,20270404T020000,20280402T020000,",
                 b" 20290401T020000,20300407T020000,20310406T020000,20320404T020000,20330403T0",
                 b" 20000,20340402T020000,20350401T020000,20360406T020000,20370405T020000",
                 b"TZNAME:NZST",
                 b"TZOFFSETFROM:+1300",
                 b"TZOFFSETTO:+1200",
                 b"END:STANDARD", ])
        self.assertIn(nzst, export)

    def testOverlapping(self):
        tz = pytz.timezone("Pacific/Auckland")
        vev = SimpleVEvent()
        addProps(vev, summary = "Storytelling", uid = "1000",
                 dtstart = timezone.make_aware(dt.datetime(2016, 4, 5, 10), tz),
                 dtend   = timezone.make_aware(dt.datetime(2016, 4, 5, 11), tz),
                 rrule   = vRecur.from_ical("FREQ=WEEKLY;WKST=SU;BYDAY=TU;UNTIL=20160519"))
        span = TimeZoneSpan(vev)
        vev = SimpleVEvent()
        addProps(vev, summary = "Parade", uid = "1002",
                 dtstart = timezone.make_aware(dt.datetime(2016, 4, 5, 9), tz),
                 dtend   = timezone.make_aware(dt.datetime(2016, 4, 5, 13), tz))
        span.add(vev)
        vtz = span.createVTimeZone(tz)
        self.assertEqual(vtz['TZID'], "Pacific/Auckland")
        export = vtz.to_ical()
        nzdt = b"\r\n".join([
                 b"BEGIN:DAYLIGHT",
                 b"DTSTART;VALUE=DATE-TIME:20160925T030000",
                 b"TZNAME:NZDT",
                 b"TZOFFSETFROM:+1200",
                 b"TZOFFSETTO:+1300",
                 b"END:DAYLIGHT", ])
        self.assertIn(nzdt, export)
        nzst = b"\r\n".join([
                 b"BEGIN:STANDARD",
                 b"DTSTART;VALUE=DATE-TIME:20160403T020000",
                 b"TZNAME:NZST",
                 b"TZOFFSETFROM:+1300",
                 b"TZOFFSETTO:+1200",
                 b"END:STANDARD", ])
        self.assertIn(nzst, export)

# ------------------------------------------------------------------------------
class TestVMatch(TestCase):
    def testEmpty(self):
        match = VMatch()
        self.assertIs(match.parent, None)
        self.assertEqual(match.orphans, [])

    def testDuplicate(self):
        match = VMatch()
        vev1 = SimpleVEvent()
        addProps(vev1, summary = "Event1", uid = "1234",
                 dtstart = dt.datetime(2016, 4, 5, 9),
                 dtend   = dt.datetime(2016, 4, 5, 13))
        match.add(vev1)
        vev2 = SimpleVEvent()
        addProps(vev2, summary = "Event2", uid = "1234",
                 dtstart = dt.datetime(2016, 4, 5, 9),
                 dtend   = dt.datetime(2016, 4, 5, 13))
        with self.assertRaises(VMatch.DuplicateError):
            match.add(vev2)

# ------------------------------------------------------------------------------
class TestVEventFactory(TestCase):
    def setUp(self):
        self.factory = VEventFactory()

    def testMissingUid(self):
        props = Event()
        addProps(props, summary = "Event",
                 dtstart = dt.datetime(2016, 4, 5, 9),
                 dtend   = dt.datetime(2016, 4, 5, 13))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "Missing UID")

    def testMissingDtstamp(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart = dt.datetime(2016, 4, 5, 9),
                 dtend   = dt.datetime(2016, 4, 5, 13))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "Missing DTSTAMP")

    def testMissingDtstart(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtend   = dt.datetime(2016, 4, 5, 13),
                 dtstamp = dt.datetime(2016, 4, 5, 13))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "Missing DTSTART")

    def testDtendAndDuration(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = dt.datetime(2016, 4, 5, 9),
                 dtend    = dt.datetime(2016, 4, 5, 13),
                 dtstamp  = dt.datetime(2016, 4, 5, 13),
                 duration = dt.timedelta(2))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "Both DURATION and DTEND set")

    def testDtstartDtendTypes(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = dt.datetime(2016, 4, 5, 9),
                 dtend    = dt.date(2016, 4, 5),
                 dtstamp  = dt.datetime(2016, 4, 5, 13))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "DTSTART and DTEND types do not match")

    def testDiffTZsDtstartDtEnd(self):
        tz1 = pytz.timezone("Pacific/Auckland")
        tz2 = pytz.timezone("Australia/Sydney")
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = timezone.make_aware(dt.datetime(2016, 6, 1, 7), tz1),
                 dtend    = timezone.make_aware(dt.datetime(2016, 6, 1, 8), tz2),
                 dtstamp  = timezone.make_aware(dt.datetime(2016, 6, 1, 8), tz2))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "DTSTART.timezone != DTEND.timezone")

    def testMultipleRRules(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = dt.datetime(2016, 4, 5, 9),
                 dtend    = dt.datetime(2016, 4, 5, 13),
                 dtstamp  = dt.datetime(2016, 4, 5, 13),
                 rrule   = vRecur.from_ical("FREQ=WEEKLY;WKST=SU;BYDAY=MO,TU,WE,TH,FR"))
        props.add('rrule', vRecur.from_ical("FREQ=MONTHLY;WKST=SU;BYDAY=-1SA,-1SU"))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception), "Multiple RRULEs")

    def testDiffTZsDtstartRecurrenceId(self):
        tz1 = pytz.timezone("Pacific/Auckland")
        tz2 = pytz.timezone("Australia/Sydney")
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = timezone.make_aware(dt.datetime(2016, 6, 1, 7), tz1),
                 dtend    = timezone.make_aware(dt.datetime(2016, 6, 1, 10), tz1),
                 dtstamp  = timezone.make_aware(dt.datetime(2016, 6, 1, 8), tz2))
        props.add('RECURRENCE-ID', timezone.make_aware(dt.datetime(2016, 6, 1, 8), tz2))
        with self.assertRaises(CalendarTypeError) as expected:
            self.factory.makeFromProps(props, None)
        self.assertEqual(str(expected.exception),
                         "DTSTART.timezone != RECURRENCE-ID.timezone")

    def testWholeDayEvent(self):
        props = Event(summary = "Event", uid = "1234")
        props.add('dtstart', dt.date(2017, 6, 5))
        props.add('dtend',   dt.date(2017, 6, 6))
        props.add('dtstamp', vDt(timezone.make_aware(dt.datetime(2016, 6, 7, 8))))
        vev = self.factory.makeFromProps(props, None)
        self.assertIs(type(vev), SimpleVEvent)

    def test1hrDuration(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = timezone.make_aware(dt.datetime(2016, 6, 1, 7)),
                 duration = dt.timedelta(hours=1),
                 dtstamp  = timezone.make_aware(dt.datetime(2016, 6, 1, 8)))
        vev = self.factory.makeFromProps(props, None)
        self.assertIs(type(vev), SimpleVEvent)
        self.assertEqual(vev['DTSTART'], timezone.make_aware(dt.datetime(2016, 6, 1, 7)))
        self.assertEqual(vev['DTEND'],   timezone.make_aware(dt.datetime(2016, 6, 1, 8)))

    def testNoDuration(self):
        props = Event()
        addProps(props, summary = "Event", uid = "1234",
                 dtstart  = dt.date(2016, 6, 1),
                 dtstamp  = timezone.make_aware(dt.datetime(2016, 6, 1, 8)))
        vev = self.factory.makeFromProps(props, None)
        self.assertIs(type(vev), SimpleVEvent)
        self.assertEqual(vev['DTSTART'], dt.date(2016, 6, 1))
        self.assertEqual(vev['DTEND'],   dt.date(2016, 6, 2))

    def testUnsupportedPageType(self):
        page = Page(title = "Event", slug = "event")
        with self.assertRaises(CalendarTypeError) as expected:
            vev = self.factory.makeFromPage(page)
        self.assertEqual(str(expected.exception), "Unsupported page type")

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# Generated by Django 2.1.5 on 2019-02-20 22:52

import datetime
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import ls.joyous.fields
import ls.joyous.models.events
import timezone_field.fields
import wagtail.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('joyous', '0012_auto_20181231_1827'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='calendarpage',
            options={'verbose_name': 'multiday event page', 'verbose_name_plural': 'multiday event pages'},
        ),
        migrations.AlterModelOptions(
            name='cancellationpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'cancellation', 'verbose_name_plural': 'cancellations'},
        ),
        migrations.AlterModelOptions(
            name='eventcategory',
            options={'ordering': ['name'], 'verbose_name': 'event category', 'verbose_name_plural': 'event categories'},
        ),
        migrations.AlterModelOptions(
            name='extrainfopage',
            options={'default_manager_name': 'objects', 'verbose_name': 'extra event information', 'verbose_name_plural': 'extra event information'},
        ),
        migrations.AlterModelOptions(
            name='multidayeventpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'multiday event page', 'verbose_name_plural': 'multiday event pages'},
        ),
        migrations.AlterModelOptions(
            name='multidayrecurringeventpage',
            options={'verbose_name': 'multiday recurring event page', 'verbose_name_plural': 'multiday recurring event pages'},
        ),
        migrations.AlterModelOptions(
            name='postponementpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'postponement', 'verbose_name_plural': 'postponements'},
        ),
        migrations.AlterModelOptions(
            name='recurringeventpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'recurring event page', 'verbose_name_plural': 'recurring event pages'},
        ),
        migrations.AlterModelOptions(
            name='simpleeventpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'event page', 'verbose_name_plural': 'event pages'},
        ),
        migrations.AlterField(
            model_name='calendarpage',
            name='default_view',
            field=models.CharField(choices=[('L', 'List View'), ('W', 'Weekly View'), ('M', 'Monthly View')], default='M', max_length=15, verbose_name='default view'),
        ),
        migrations.AlterField(
            model_name='calendarpage',
            name='intro',
            field=wagtail.core.fields.RichTextField(blank=True, verbose_name='intro'),
        ),
        migrations.AlterField(
            model_name='calendarpage',
            name='view_choices',
            field=ls.joyous.fields.MultipleSelectField(blank=True, choices=[('L', 'List View'), ('W', 'Weekly View'), ('M', 'Monthly View')], default=['L', 'W', 'M'], verbose_name='view choices'),
        ),
        migrations.AlterField(
            model_name='cancellationpage',
            name='cancellation_details',
            field=wagtail.core.fields.RichTextField(blank=True, help_text='Why was the event cancelled?', verbose_name='details'),
        ),
        migrations.AlterField(
            model_name='cancellationpage',
            name='cancellation_title',
            field=models.CharField(blank=True, help_text='Show in place of cancelled event (Leave empty to show nothing)', max_length=255, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='cancellationpage',
            name='overrides',
            field=models.ForeignKey(help_text='The recurring event that we are updating.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='joyous.RecurringEventPage', verbose_name='overrides'),
        ),
        migrations.AlterField(
            model_name='eventcategory',
            name='code',
            field=models.CharField(max_length=4, unique=True, verbose_name='code'),
        ),
        migrations.AlterField(
            model_name='eventcategory',
            name='name',
            field=models.CharField(max_length=80, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='extrainfopage',
            name='extra_information',
            field=wagtail.core.fields.RichTextField(help_text='Information just for this date', verbose_name='extra information'),
        ),
        migrations.AlterField(
            model_name='extrainfopage',
            name='extra_title',
            field=models.CharField(blank=True, help_text='A more specific title for this occurence (optional)', max_length=255, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='extrainfopage',
            name='overrides',
            field=models.ForeignKey(help_text='The recurring event that we are updating.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='joyous.RecurringEventPage', verbose_name='overrides'),
        ),
        migrations.AlterField(
            model_name='grouppage',
            name='content',
            field=wagtail.core.fields.RichTextField(blank=True, default='', help_text='An area of text for whatever you like', verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='joyous.EventCategory', verbose_name='category'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='date_from',
            field=models.DateField(default=datetime.date.today, verbose_name='start date'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='date_to',
            field=models.DateField(default=datetime.date.today, verbose_name='end date'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='details',
            field=wagtail.core.fields.RichTextField(blank=True, verbose_name='details'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage', verbose_name='group page'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='location',
            field=models.CharField(blank=True, max_length=255, verbose_name='location'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='time_from',
            field=models.TimeField(blank=True, null=True, verbose_name='start time'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='time_to',
            field=models.TimeField(blank=True, null=True, verbose_name='end time'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='tz',
            field=timezone_field.fields.TimeZoneField(default=ls.joyous.models.events._get_default_timezone, verbose_name='time zone'),
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='website',
            field=models.URLField(blank=True, verbose_name='website'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='joyous.EventCategory', verbose_name='category'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='date',
            field=models.DateField(verbose_name='date'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='details',
            field=wagtail.core.fields.RichTextField(blank=True, verbose_name='details'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage', verbose_name='group page'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='location',
            field=models.CharField(blank=True, max_length=255, verbose_name='location'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='postponement_title',
            field=models.CharField(help_text='The title for the postponed event', max_length=255, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='time_from',
            field=models.TimeField(blank=True, null=True, verbose_name='start time'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='time_to',
            field=models.TimeField(blank=True, null=True, verbose_name='end time'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='website',
            field=models.URLField(blank=True, verbose_name='website'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='joyous.EventCategory', verbose_name='category'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='details',
            field=wagtail.core.fields.RichTextField(blank=True, verbose_name='details'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage', verbose_name='group page'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='location',
            field=models.CharField(blank=True, max_length=255, verbose_name='location'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='num_days',
            field=models.IntegerField(default=1, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(99)], verbose_name='number of days'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='repeat',
            field=ls.joyous.fields.RecurrenceField(verbose_name='repeat'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='time_from',
            field=models.TimeField(blank=True, null=True, verbose_name='start time'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='time_to',
            field=models.TimeField(blank=True, null=True, verbose_name='end time'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='tz',
            field=timezone_field.fields.TimeZoneField(default=ls.joyous.models.events._get_default_timezone, verbose_name='time zone'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='website',
            field=models.URLField(blank=True, verbose_name='website'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='category',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='joyous.EventCategory', verbose_name='category'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='date',
            field=models.DateField(default=datetime.date.today, verbose_name='date'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='details',
            field=wagtail.core.fields.RichTextField(blank=True, verbose_name='details'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage', verbose_name='group page'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='image',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='location',
            field=models.CharField(blank=True, max_length=255, verbose_name='location'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='time_from',
            field=models.TimeField(blank=True, null=True, verbose_name='start time'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='time_to',
            field=models.TimeField(blank=True, null=True, verbose_name='end time'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='tz',
            field=timezone_field.fields.TimeZoneField(default=ls.joyous.models.events._get_default_timezone, verbose_name='time zone'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='website',
            field=models.URLField(blank=True, verbose_name='website'),
        ),
    ]



# ------------------------------------------------------------------------------
# Joyous models
# ------------------------------------------------------------------------------
import datetime as dt
from django.dispatch import receiver
from wagtail.admin.signals import init_new_page
from .models import RecurringEventPage, EventExceptionBase

# ------------------------------------------------------------------------------
# Recieve Signals
# The hook 'before_create_page' occurs too early, the page is not yet created
# so can't be modified.  The hook 'after_create_page' occurs too late, it's run
# after the form is POSTed.  
@receiver(init_new_page)
def identifyExpectantParent(sender, **kwargs):
    page = kwargs.get('page')
    parent = kwargs.get('parent')
    if (isinstance(page, EventExceptionBase) and
        isinstance(parent, RecurringEventPage) and
        not page.overrides):
        page._copyFieldsFromParent(parent)

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Reschedule Multiday Event Page
# ------------------------------------------------------------------------------
import sys
import pytz
import datetime as dt
from django.test import RequestFactory
from django_bs_test import TestCase
from django.contrib.auth.models import User
from django.utils import timezone
from wagtail.core.models import Page
from wagtail.tests.utils.form_data import nested_form_data, rich_text
from ls.joyous.models import (GeneralCalendarPage,
        MultidayRecurringEventPage, RescheduleMultidayEventPage)
from ls.joyous.utils.recurrence import Recurrence, WEEKLY, MONTHLY, MO, TU, WE, FR
from .testutils import freeze_timetz, getPage

# ------------------------------------------------------------------------------
class Test(TestCase):
    def setUp(self):
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('j', 'j@joy.test', 's3(r3t')
        self.calendar = GeneralCalendarPage(owner = self.user,
                                            slug  = "events",
                                            title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = MultidayRecurringEventPage(slug = "test-session",
                                        title = "Test Session",
                                        repeat = Recurrence(dtstart=dt.date(1990,1,2),
                                                            freq=WEEKLY,
                                                            byweekday=[TU],
                                                            until=dt.date(1990,3,29)),
                                        num_days  = 3,
                                        time_from = dt.time(10),
                                        time_to   = dt.time(16,30))
        self.calendar.add_child(instance=self.event)
        self.postponement = RescheduleMultidayEventPage(owner = self.user,
                                             overrides = self.event,
                                             except_date = dt.date(1990,1,9),
                                             postponement_title   = "Delayed Start Session",
                                             date      = dt.date(1990,1,9),
                                             num_days  = 3,
                                             time_from = dt.time(13),
                                             time_to   = dt.time(19,30))
        self.event.add_child(instance=self.postponement)
        self.postponement.save_revision().publish()

    def testWhen(self):
        self.assertEqual(self.postponement.when,
                         "Tuesday 9th of January 1990 for 3 days "
                         "starting at 1pm finishing at 7:30pm")

    @freeze_timetz("1990-02-08 18:00")
    def testStatus(self):
        postponement = RescheduleMultidayEventPage(owner = self.user,
                                             overrides = self.event,
                                             except_date = dt.date(1990,2,6),
                                             postponement_title   = "Quick Session",
                                             date      = dt.date(1990,2,6),
                                             num_days  = 1,
                                             time_from = dt.time(13),
                                             time_to   = dt.time(19,30))
        self.event.add_child(instance=postponement)
        postponement.save_revision().publish()
        self.assertIsNone(self.event.status)

    def testRemoveContentPanels(self):
        RescheduleMultidayEventPage._removeContentPanels(["tz", "location"])
        RescheduleMultidayEventPage._removeContentPanels("website")
        removed = ("tz", "location", "website")
        panels = RescheduleMultidayEventPage.content_panels
        self.assertFalse(any(field in removed
                             for panel in panels
                             for field in panel.required_fields()))

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



#!/usr/bin/env python
import os
import sys


if __name__ == '__main__':
    from django.core.management import execute_from_command_line
    os.environ['DJANGO_SETTINGS_MODULE'] = 'ls.joyous.tests.settings'
    os.environ['PYTHONSTARTUP'] = __file__
    argv = sys.argv[:1] + ['shell'] + sys.argv[1:]
    execute_from_command_line(argv)

elif __name__ == "builtins":
    # useful once we are in the shell
    import datetime as dt
    from django.utils import timezone
    from django.conf import settings
    import pytz
    import pprint
    import ls.joyous
    from ls.joyous.models import *
    from ls.joyous.utils.recurrence import *
    from ls.joyous.utils import manythings, telltime
    from ls.joyous.formats import ical
    L = list
    sys.displayhook = pprint.pprint
    sys.__interactivehook__()
    timezone.activate("Pacific/Auckland")




# ------------------------------------------------------------------------------
# vtimezone functions copied from https://github.com/pimutils/khal
# ------------------------------------------------------------------------------
# Copyright (c) 2013-2017 Christian Geier et al.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import datetime as dt
import icalendar
import pytz

def to_naive_utc(dtime):
    """convert a datetime object to UTC and than remove the tzinfo, if
    datetime is naive already, return it
    """
    if not hasattr(dtime, 'tzinfo') or dtime.tzinfo is None:
        return dtime

    dtime_utc = dtime.astimezone(pytz.UTC)
    dtime_naive = dtime_utc.replace(tzinfo=None)
    return dtime_naive

def create_timezone(tz, first_date=None, last_date=None):
    """
    create an icalendar vtimezone from a pytz.tzinfo object

    :param tz: the timezone
    :type tz: pytz.tzinfo
    :param first_date: the very first datetime that needs to be included in the
    transition times, typically the DTSTART value of the (first recurring)
    event
    :type first_date: datetime.datetime
    :param last_date: the last datetime that needs to included, typically the
    end of the (very last) event (of a recursion set)
    :returns: timezone information
    :rtype: icalendar.Timezone()

    we currently have a problem here:

       pytz.timezones only carry the absolute dates of time zone transitions,
       not their RRULEs. This will a) make for rather bloated VTIMEZONE
       components, especially for long recurring events, b) we'll need to
       specify for which time range this VTIMEZONE should be generated and c)
       will not be valid for recurring events that go into eternity.

    Possible Solutions:

    As this information is not provided by pytz at all, there is no
    easy solution, we'd really need to ship another version of the OLSON DB.

    """
    if isinstance(tz, pytz.tzinfo.StaticTzInfo):
        return _create_timezone_static(tz)

    first_date = dt.datetime.today() if not first_date else to_naive_utc(first_date)
    last_date = dt.datetime.today() if not last_date else to_naive_utc(last_date)
    timezone = icalendar.Timezone()
    timezone.add('TZID', tz)

    # This is not a reliable way of determining if a transition is for
    # daylight savings.
    # From 1927 to 1941 New Zealand had GMT+11:30 (NZ Mean Time) as standard
    # and GMT+12:00 (NZ Summer Time) as daylight savings time.
    # From 1941 GMT+12:00 (NZ Standard Time) became standard time.
    # So NZST (NZ Summer/Standard Time) can refer to standard or daylight
    # savings time.  And this code depends on the random order the _tzinfos
    # are returned.
    # dst = {
    #     one[2]: 'DST' in two.__repr__()
    #     for one, two in iter(tz._tzinfos.items())
    # }
    # bst = {
    #     one[2]: 'BST' in two.__repr__()
    #     for one, two in iter(tz._tzinfos.items())
    # }
    # ...
    #   if dst[name] or bst[name]:

    # looking for the first and last transition time we need to include
    first_num, last_num = 0, len(tz._utc_transition_times) - 1
    first_tt = tz._utc_transition_times[0]
    last_tt = tz._utc_transition_times[-1]
    for num, transtime in enumerate(tz._utc_transition_times):
        if transtime > first_tt and transtime < first_date:
            first_num = num
            first_tt = transtime
        if transtime < last_tt and transtime > last_date:
            last_num = num
            last_tt = transtime

    timezones = dict()
    for num in range(first_num, last_num + 1):
        name = tz._transition_info[num][2]
        if name in timezones:
            ttime = tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None)
            if 'RDATE' in timezones[name]:
                timezones[name]['RDATE'].dts.append(
                    icalendar.prop.vDDDTypes(ttime))
            else:
                timezones[name].add('RDATE', ttime)
            continue

        if tz._transition_info[num][1]:
            subcomp = icalendar.TimezoneDaylight()
        else:
            subcomp = icalendar.TimezoneStandard()

        subcomp.add('TZNAME', tz._transition_info[num][2])
        subcomp.add(
            'DTSTART',
            tz.fromutc(tz._utc_transition_times[num]).replace(tzinfo=None))
        subcomp.add('TZOFFSETTO', tz._transition_info[num][0])
        subcomp.add('TZOFFSETFROM', tz._transition_info[num - 1][0])
        timezones[name] = subcomp

    for subcomp in timezones.values():
        timezone.add_component(subcomp)

    return timezone


def _create_timezone_static(tz):
    """create an icalendar vtimezone from a pytz.tzinfo.StaticTzInfo

    :param tz: the timezone
    :type tz: pytz.tzinfo.StaticTzInfo
    :returns: timezone information
    :rtype: icalendar.Timezone()
    """
    timezone = icalendar.Timezone()
    timezone.add('TZID', tz)
    subcomp = icalendar.TimezoneStandard()
    subcomp.add('TZNAME', tz)
    subcomp.add('DTSTART', dt.datetime(1601, 1, 1))
    subcomp.add('RDATE', dt.datetime(1601, 1, 1))
    subcomp.add('TZOFFSETTO', tz._utcoffset)
    subcomp.add('TZOFFSETFROM', tz._utcoffset)
    timezone.add_component(subcomp)
    return timezone



# ------------------------------------------------------------------------------
# Joyous initialization
# ------------------------------------------------------------------------------
name = "joyous"
try:
    from ._version import version as __version__
except ImportError:  # pragma: no cover
    __version__ = '1.1'

default_app_config = 'ls.joyous.apps.JoyousAppConfig'

# ------------------------------------------------------------------------------
# Note: Default settings
# ------------------------------------------------------------------------------
# settings.JOYOUS_HOLIDAYS = ""
# settings.JOYOUS_GROUP_SELECTABLE = False
# settings.JOYOUS_GROUP_MODEL = "joyous.GroupPage"
# settings.JOYOUS_TIME_INPUT = "24"
# settings.JOYOUS_EVENTS_PER_PAGE = 25
# settings.JOYOUS_THEME_CSS = ""
# settings.JOYOUS_RSS_FEED_IMAGE = "joyous/img/logo.png"
# settings.JOYOUS_UPCOMING_INCLUDES_STARTED = False



# ------------------------------------------------------------------------------
# RSS Feed Export Handler
# ------------------------------------------------------------------------------
import datetime as dt
from django.http import HttpResponse
from django.conf import settings
from django.template import loader
from django.templatetags.static import static
from ..models import (CalendarPage, SimpleEventPage, MultidayEventPage,
        RecurringEventPage, ExtraInfoPage, CancellationPage, PostponementPage)
from feedgen.feed import FeedGenerator
from feedgen.entry import FeedEntry
from .errors import CalendarTypeError

# ------------------------------------------------------------------------------
class RssHandler:
    """Serve a RSS Feed"""
    def serve(self, page, request, *args, **kwargs):
        try:
            feed = CalendarFeed.fromPage(page, request)
        except CalendarTypeError:
            return None
        response = HttpResponse(feed.rss_str(),
                                content_type='application/xml; charset=utf-8')
        return response

# ------------------------------------------------------------------------------
def fullUrl(url, page, request):
    """Convert a relative url to a full url"""
    siteId, root, path = page.get_url_parts(request)
    if not url.startswith(root):
        url = root + url
    return url

# ------------------------------------------------------------------------------
class CalendarFeed(FeedGenerator):
    """Produce a feed of upcoming events"""
    imagePath = static("joyous/img/logo.png")

    @classmethod
    def fromPage(cls, page, request):
        if isinstance(page, CalendarPage):
            return cls._fromCalendarPage(page, request)
        else:
            raise CalendarTypeError("Unsupported input page")

    @classmethod
    def _fromCalendarPage(cls, page, request):
        feed = cls()
        feed.title(page.title)
        feed.link(href=page.get_full_url(request))
        feed.author(name=page.owner.get_full_name())
        feed.description(page.intro or page.title)
        feed.generator("ls.joyous")
        imagePath = getattr(settings, "JOYOUS_RSS_FEED_IMAGE", cls.imagePath)
        feed.image(url=fullUrl(imagePath, page, request))

        for thisEvent in page._getUpcomingEvents(request):
            entry = cls._makeFromEvent(thisEvent, request)
            feed.entry(entry)
        return feed

    @classmethod
    def _makeFromEvent(cls, thisEvent, request):
        page = thisEvent.page
        if isinstance(page, (SimpleEventPage, MultidayEventPage,
                             RecurringEventPage, PostponementPage)):
            return EventEntry.fromEvent(thisEvent, request)
        elif isinstance(page, ExtraInfoPage):
            return ExtraInfoEntry.fromEvent(thisEvent, request)
        # No Cancellations are returned from _getUpcomingEvents

# ------------------------------------------------------------------------------
class EventEntry(FeedEntry):
    template = "joyous/formats/rss_entry.xml"

    @classmethod
    def fromEvent(cls, thisEvent, request):
        page = thisEvent.page
        entry = cls()
        entry.title(thisEvent.title)
        url = fullUrl(thisEvent.url, page, request)
        entry.link(href=url)
        entry.guid(url, permalink=True)
        entry.setDescription(thisEvent, request)
        entry.setCategory(page)
        entry.setImage(page, request)
        entry.author(name=page.owner.get_full_name())
        entry.published(page.first_published_at)
        entry.updated(page.last_published_at)
        return entry

    def setDescription(self, thisEvent, request):
        page = thisEvent.page
        tmpl = loader.get_template(self.template)
        ctxt = {'event':   page,
                'title':   thisEvent.title,
                'details': page.details,
                'request': request}
        descr = tmpl.render(ctxt, request)
        self.description(descr)
        # NOTE: feedgen will escape the HTML and that is a GOOD THING

    def setCategory(self, page):
        category = page.category
        if category:
            self.category(term=category.name)

    def setImage(self, page, request):
        image = page.image
        if image:
            ren = image.get_rendition("width-350|format-png")
            self.enclosure(url=fullUrl(ren.url, page, request),
                           length=str(len(ren.file)),
                           type="image/png")

# ------------------------------------------------------------------------------
class ExtraInfoEntry(EventEntry):
    template = "joyous/formats/rss_extra_info_entry.xml"

    def setDescription(self, thisEvent, request):
        page = thisEvent.page
        tmpl = loader.get_template(self.template)
        ctxt = {'event':   page,
                'title':   thisEvent.title,
                'extra_information': page.extra_information,
                'details': page.overrides.details,
                'request': request}
        descr = tmpl.render(ctxt, request)
        self.description(descr)

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
